<?php
/*
Plugin Name: Display graph by chartjs
Plugin URI: https://tarovlog.com/display-graph-by-chartjs/
Description: Chart JS を使ったグラフ描画プラグイン
Version: 1.1.2
Author: tarovlog
Author URI: https://tarovlog.com/
License: GPL2
*/

/*
 * js, css の読み込み
 *************************************/
function add_my_style_and_scripts() {
    wp_enqueue_script('my-chartjs', plugins_url() . '/display-graph-by-chartjs/js/Chart.min.js', null, '2.8.0', false);
    wp_enqueue_script('my-chartjs-plugin', plugins_url() . '/display-graph-by-chartjs/js/chartjs-plugin-labels.js', null, '2.8.0', false);
}
add_action('wp_enqueue_scripts', 'add_my_style_and_scripts');

/*
 * graph用のClass
 *************************************/
class Dgbcshowgraph {

	function __construct($type, $title, $x_data, $y_data, $y_top, $y_label, $height, $bordercolor, $backgroundcolor, $unit, $labelson, $comma)	{
		$this->type = $type;
    $this->title = $title;
    $this->x_data = $this->makelist($x_data);
    $this->y_data = $y_data;
    $this->y_top = $y_top;
    $this->y_label = $y_label;
    $this->height = $height;
    $this->bordercolor = $this->makelistcolor($bordercolor);
    $this->backgroundcolor = $this->makelistcolor($backgroundcolor);
    $this->shuffled = str_shuffle('abcdef');
    $this->unit = $unit;
    $this->labelson = $labelson;
    $this->comma = $comma;
	}

  function checker() {
    echo '<pre>';
    echo $this->type;
    echo $this->title;
    echo $this->x_data;
    echo $this->y_data;
    echo $this->y_top;
    echo $this->y_label;
    echo $this->height;
    echo $this->bordercolor;
    echo $this->backgroundcolor;
    echo $this->shuffled;
    echo $this->unit;
    echo $this->labelson;
    echo $this->comma;
    echo '</pre>';
  }

  function head() {
    $html = '';
    $html .= "<div style='height: {$this->height}px'>";
    $html .= "<canvas id='{$this->shuffled}'></canvas>";
    $html .= "</div>";
    $html .= "<a href='https://tarovlog.com/display-graph-by-chartjs/' style='display: inline-block;font-size: 12px;font-style: italic;margin-bottom: 20px;'>Plugin by tarovlog.com</a>";
    return $html;
  }

  function makelist($data) {
    $arr = explode(',', str_replace(' ', '', $data));
    $output = '[\''.join("','", $arr).'\']';
    return $output;
  }

  function makelistcolor($data) {
    $arr = explode(',', str_replace(' ', '', $data));
    if (count($arr) == 1) {
      $output = '\''.join("','", $arr).'\'';
    } else {
      $output = '[\''.join("','", $arr).'\']';
    }
    return $output;
  }

	function oresen_graph() {
    $html = '';
    $html .= <<<EOT
    <script>
    var ctx = document.getElementById('{$this->shuffled}').getContext('2d');
    var myLineChart = new Chart(ctx, {
      type: '{$this->type}',
      data: {
        labels: {$this->x_data},
        datasets: [{
          label: '{$this->y_label}',
          data: [{$this->y_data}],
          borderColor: {$this->bordercolor},
          backgroundColor: 'rgb(0,0,0,0)'
        }],
      },
      options: {
        responsive: true,
        maintainAspectRatio: false,
        tooltips: {
          callbacks: {
            label: function(tooltipItem, data) {
              var value = data.datasets[0].data[tooltipItem.index];
              if(parseInt(value) >= 1000 && {$this->comma}){
                return value.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
              } else {
                return value;
              }
            }
          } // end callbacks:
        }, //end tooltips
        title: {
          display: true,
          text: '{$this->title}'
        },
        scales: {
          yAxes: [{
            ticks: {
              suggestedMax: {$this->y_top},
              suggestedMin: 0,
              callback: function(value, index, values) {
                if(parseInt(value) >= 1000 && {$this->comma}){
                  return value.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
                } else {
                  return value;
                }
              }
            }
          }]
        },
      }
    });
    </script>
EOT;
  return $html;
	}

  function en_graph() {
    $html = '';
    $html .= <<<EOT
    <script>
    var ctx = document.getElementById('{$this->shuffled}').getContext('2d');
    var myLineChart = new Chart(ctx, {
      type: '{$this->type}',
      data: {
        labels: {$this->x_data},
        datasets: [{
          data: [{$this->y_data}],
          backgroundColor: {$this->backgroundcolor}
        }],
      },
      options: {
        responsive: true,
        maintainAspectRatio: false,
        title: {
          display: true,
          text: '{$this->title}'
        },
        plugins: {
          labels: {
            render: function (args) {
              return {$this->labelson} ? args.label +'\\n'+ args.value + '{$this->unit}' : '' ;
            },
            position: 'default',
            fontSize: 12,
            fontColor: '#333',
            overlap: false,
          }
        }
      }
    });
    </script>
EOT;
  return $html;
	}

  function bar_graph() {
    $html = '';
    $html .= <<<EOT
    <script>
    var ctx = document.getElementById('{$this->shuffled}');
    var myLineChart = new Chart(ctx, {
      type: '{$this->type}',
      data: {
        labels: {$this->x_data},
        datasets: [{
          label: '{$this->y_label}',
          data: [{$this->y_data}],
          backgroundColor: {$this->backgroundcolor},
          borderColor: {$this->bordercolor}
        }]
      },
      options: {
        responsive: true,
        maintainAspectRatio: false,
        title: {
          display: true,
          text: '{$this->title}'
        },
        plugins: {
          labels: {
            render: function (args) {
              return {$this->labelson} ? args.label +'\\n'+ args.value + '{$this->unit}' : '' ;
            },
          }
        },
        scales: {
          yAxes: [{
            ticks: {
              suggestedMax: {$this->y_top},
              suggestedMin: 0,
            }
          }]
        }
      }
    });
    </script>
EOT;
  return $html;
  }

}

/*
 * plot single data shortcode
 *************************************/
function dgbc_display_graph_single($atts) {
    wp_enqueue_script('my-chartjs');
    wp_enqueue_script('my-chartjs-plugin');

    // set parameter
    $type = empty($atts['type']) ? 'line' : $atts['type'];
    $title = empty($atts['title']) ? '' : $atts['title'];
    $x_data = empty($atts['x_data']) ? "1月, 2月, 3月, 4月, 5月" : $atts['x_data'];
    $y_data = empty($atts['y_data']) ? "10, 30, 60, 150, 280" : $atts['y_data'];
    $y_top = empty($atts['y_top']) ? '100' : $atts['y_top'];
    $y_label = empty($atts['y_label']) ? '' : $atts['y_label'];
    $height = empty($atts['height']) ? '300' : $atts['height'];
    $bordercolor = empty($atts['bordercolor']) ? '#778899' : $atts['bordercolor'];
    $backgroundcolor = empty($atts['backgroundcolor']) ? 'rgb(0,0,0,0)' : $atts['backgroundcolor'];
    $unit = empty($atts['unit']) ? '' : $atts['unit'];
    $labelson = isset($atts['labelson']) && $atts['labelson'] == 'true' ? "true" : "false";
    $comma = isset($atts['comma']) && $atts['comma'] == 'true' ? "true" : "false";

    // chart type
    if (empty($type)) {
        return 'グラフの種類を指定してください。';
    }
    // input (x, y) data
    if (empty($x_data) || empty($y_data)) {
        return 'x_dataまたはy_dataデータを入力してください。';
    }
    
    // $html = '<script src="https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.8.0/Chart.min.js"></script>';
    // $html .= '<script src="https://cdn.jsdelivr.net/gh/emn178/chartjs-plugin-labels/src/chartjs-plugin-labels.js"></script>';
    // $graph->checker();
    $graph = new Dgbcshowgraph($type, $title, $x_data, $y_data, $y_top, $y_label, $height, $bordercolor, $backgroundcolor, $unit, $labelson, $comma);
    $html .= $graph->head();
    if ($type == 'line') {
      $html .= $graph->oresen_graph();
    } else if ($type == 'pie') {
      $html .= $graph->en_graph();
    } else if ($type == 'bar') {
      $html .= $graph->bar_graph();
    }

    return $html;
}
add_shortcode( 'display_graph_single', 'dgbc_display_graph_single');


/*
 * 2つgraph表示用のClass
 *************************************/
class Dgbcshowgraphmulti {

	function __construct($type, $height, $title, $x_data, $label_data1, $y_data1, $color_data1, $label_data2, $y_data2, $color_data2, $data1_max, $data1_step, $data2_max, $data2_step, $comma)	{
		$this->type = $type;
    $this->title = $title;
    $this->x_data = $this->makelist($x_data);
    $this->label_data1 = $label_data1;
    $this->y_data1 = $this->makelist($y_data1);
    $this->color_data1 = $color_data1;
    $this->label_data2 = $label_data2;
    $this->y_data2 = $this->makelist($y_data2);
    $this->color_data2 = $color_data2;
    $this->data1_max = $data1_max;
    $this->data1_step = $data1_step;
    $this->data2_max = $data2_max;
    $this->data2_step = $data2_step;    
    $this->shuffled = str_shuffle('abcdef');
    $this->height = $height;
    $this->comma = $comma;
    // $this->unit = $unit;
    // $this->labelson = $labelson;
	}

  function checker() {
    echo '<pre>';
    echo $this->title;
    echo $this->x_data;
    echo $this->label_data1;
    echo $this->y_data1;
    echo $this->color_data1;
    echo $this->label_data2;
    echo $this->y_data2;
    echo $this->color_data2;
    echo $this->data1_max;
    echo $this->data1_step;
    echo $this->data2_max;
    echo $this->data2_step;
    echo $this->shuffled;
    echo $this->comma;
    // echo $this->unit;
    // echo $this->labelson;
    echo '</pre>';
  }
  
  function head() {
    $html = '';
    $html .= "<div style='height: {$this->height}px'>";
    $html .= "<canvas id='{$this->shuffled}'></canvas>";
    $html .= "</div>";
    $html .= "<a href='https://tarovlog.com/display-graph-by-chartjs/' style='display: inline-block;font-size: 12px;font-style: italic;margin-bottom: 20px;'>Plugin by tarovlog.com</a>";
    return $html;
  }

  function makelist($data) {
    $arr = explode(',', str_replace(' ', '', $data));
    $output = '[\''.join("','", $arr).'\']';
    return $output;
  }

  function makelistcolor($data) {
    $arr = explode(',', str_replace(' ', '', $data));
    if (count($arr) == 1) {
      $output = '\''.join("','", $arr).'\'';
    } else {
      $output = '[\''.join("','", $arr).'\']';
    }
    return $output;
  }

  function oresen_and_bar_graph() {
    $html = '';
    $html .= <<<EOT
    <script>
    var ctx = document.getElementById('{$this->shuffled}');
    var myLineChart = new Chart(ctx, {
      type: 'bar',
      data: {
        labels: {$this->x_data},
        datasets: [{
          label: '{$this->label_data1}',
          type: "line",
          fill: false,
          data: {$this->y_data1},
          borderColor: '{$this->color_data1}',
          backgroundColor: '{$this->color_data1}',
          yAxisID: "y-axis-1",
        }, {
          label: '{$this->label_data2}',
          data: {$this->y_data2},
          borderColor: '{$this->color_data2}',
          backgroundColor: '{$this->color_data2}',
          yAxisID: "y-axis-2",
      }]
      },
      options: {
        tooltips: {
          mode: 'nearest',
          intersect: false,
          callbacks: {
            label: function(tooltipItem, data) {
              var dataIndex = tooltipItem.datasetIndex;
              var value = data.datasets[dataIndex].data[tooltipItem.index];
              if(parseInt(value) >= 1000 && {$this->comma}){
                return value.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
              } else {
                return value;
              }
            }
          }, // end callbacks
        },
        responsive: true,
        maintainAspectRatio: false,
        title: {
          display: true,
          text: '{$this->title}'
        },
        plugins: {
          labels: {
            render: function (args) {
              return '' ;
              // return true ? args.value + '{$this->unit}' : '' ;
            },
          }
        },
        scales: {
          yAxes: [{
            id: "y-axis-1",
            type: "linear",
            position: "left",
            ticks: {
                max: {$this->data1_max},
                min: 0,
                stepSize: {$this->data1_step},
                callback: function(value, index, values) {
                  if(parseInt(value) >= 1000 && {$this->comma}){
                    return value.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
                  } else {
                    return value;
                  }
                }
            },
          }, 
          {
            id: "y-axis-2",
            type: "linear",
            position: "right",
            ticks: {
                max: {$this->data2_max},
                min: 0,
                stepSize: {$this->data2_step},
                callback: function(value, index, values) {
                  if(parseInt(value) >= 1000 && {$this->comma}){
                    return value.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
                  } else {
                    return value;
                  }
                }
            },
            gridLines: {
                drawOnChartArea: false,
            },
        }],
        }
      }
    });
    </script>
EOT;
  return $html;
  }

  function oresen_and_oresen_graph() {
    $html = '';
    $html .= <<<EOT
    <script>
    var ctx = document.getElementById('{$this->shuffled}');
    var myLineChart = new Chart(ctx, {
      type: 'line',
      data: {
        labels: {$this->x_data},
        datasets: [{
          label: '{$this->label_data1}',
          type: "line",
          fill: false,
          data: {$this->y_data1},
          borderColor: '{$this->color_data1}',
          backgroundColor: 'rgba(255,255,255,0)',
          pointBackgroundColor: '{$this->color_data1}',
        }, {
          label: '{$this->label_data2}',
          data: {$this->y_data2},
          borderColor: '{$this->color_data2}',
          backgroundColor: 'rgba(255,255,255,0)',
          pointBackgroundColor: '{$this->color_data2}',
      }]
      },
      options: {
        tooltips: {
          mode: 'nearest',
          intersect: false,
          callbacks: {
            label: function(tooltipItem, data) {
              var dataIndex = tooltipItem.datasetIndex;
              var value = data.datasets[dataIndex].data[tooltipItem.index];
              if(parseInt(value) >= 1000 && {$this->comma}){
                return value.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
              } else {
                return value;
              }
            }
          }, // end callbacks
        }, //end tooltips
        responsive: true,
        maintainAspectRatio: false,
        title: {
          display: true,
          text: '{$this->title}'
        },
        plugins: {
          labels: {
            render: function (args) {
              return '' ;
              // return true ? args.value + '{$this->unit}' : '' ;
            },
          }
        },
        scales: {
          yAxes: [{
            // id: "y-axis-1",
            type: "linear",
            position: "left",
            ticks: {
                max: {$this->data1_max},
                min: 0,
                stepSize: {$this->data1_step},
                callback: function(value, index, values) {
                  if(parseInt(value) >= 1000 && {$this->comma}){
                    return value.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
                  } else {
                    return value;
                  }
                }
            },
          }, 
        ],
        }
      }
    });
    </script>
EOT;
  return $html;
  }

}

/*
 * plot multi data shortcode
 *************************************/
function dgbc_display_graph_multi($atts) {
    wp_enqueue_script('my-chartjs');
    wp_enqueue_script('my-chartjs-plugin');

    // set parameter
    $type = empty($atts['type']) ? 'oresen_and_bar' : $atts['type'];
    $height = empty($atts['height']) ? '300' : $atts['height'];
    $title = empty($atts['title']) ? '' : $atts['title'];
    $label_data1 = empty($atts['label_data1']) ? "data1" : $atts['label_data1'];
    $x_data = empty($atts['x_data']) ? "1月, 2月, 3月, 4月, 5月" : $atts['x_data'];
    $y_data1 = empty($atts['y_data1']) ? "100, 310, 550, 120, 90, 820, 930" : $atts['y_data1'];
    $color_data1 = empty($atts['color_data1']) ? "#9AA2EB" : $atts['color_data1'];
    $label_data2 = empty($atts['label_data2']) ? "data2" : $atts['label_data2'];
    $y_data2 = empty($atts['y_data2']) ? "22, 23, 10, 15, 40, 35, 30" : $atts['y_data2'];
    $color_data2 = empty($atts['color_data2']) ? "#FF6384" : $atts['color_data2'];
    $data1_max = empty($atts['data1_max']) ? "1000" : $atts['data1_max'];
    $data1_step = empty($atts['data1_step']) ? "100" : $atts['data1_step'];
    $data2_max = empty($atts['data2_max']) ? "200" : $atts['data2_max'];
    $data2_step = empty($atts['data2_step']) ? "20" : $atts['data2_step'];
    $comma = isset($atts['comma']) && $atts['comma'] == 'true' ? "true" : "false";

    // chart type
    if (empty($type)) {
        return 'グラフの種類を指定してください。';
    }

    $graph = new Dgbcshowgraphmulti($type, $height, $title, $x_data, $label_data1, $y_data1, $color_data1, $label_data2, $y_data2, $color_data2, $data1_max, $data1_step, $data2_max, $data2_step, $comma);
    // var_dump($graph->checker());
    $html .= $graph->head();
    if ($type == 'oresen_and_bar') {
      $html .= $graph->oresen_and_bar_graph();
    } else if ($type == 'oresen_and_oresen') {
      $html .= $graph->oresen_and_oresen_graph();
    }

    return $html;
}
add_shortcode( 'display_graph_multi', 'dgbc_display_graph_multi');

